@@ -1,17 +1,7 @@
 import warnings
 from typing import Sequence
 
-import numpy.random
-from numpy import concatenate
-from numpy import copy
-from numpy import insert
-from numpy import isscalar
-from numpy import median
-from numpy import ndim
-from numpy import newaxis
-from numpy import sort
-from numpy import vstack
-from numpy import zeros
+import numpy as np
 from scipy.integrate import odeint
 from scipy.integrate import solve_ivp
 from scipy.interpolate import interp1d
@@ -24,11 +14,14 @@ from sklearn.pipeline import Pipeline
 from sklearn.utils.validation import check_is_fitted
 
 from .differentiation import FiniteDifference
+from .feature_library import GeneralizedLibrary
 from .feature_library import PDELibrary
 from .feature_library import PolynomialLibrary
 from .feature_library import SINDyPILibrary
+from .feature_library import WeakPDELibrary
 from .optimizers import SINDyOptimizer
 from .optimizers import STLSQ
+from .utils import convert_u_dot_integral
 from .utils import drop_nan_rows
 from .utils import drop_random_rows
 from .utils import equations
@@ -310,15 +303,27 @@ class SINDy(BaseEstimator):
                 trim_last_point=trim_last_point,
             )
             self.n_control_features_ = u.shape[1]
+        if isinstance(x, list) and not multiple_trajectories:
+            raise ValueError(
+                "x is a list (assumed to be a list of trajectories), but "
+                "multiple_trajectories is set to False."
+            )
 
         if (ensemble or library_ensemble) and n_models is None:
             n_models = 20
         if ensemble and n_subset is None:
-            n_subset = x.shape[0]
+            if multiple_trajectories:
+                if x[0].ndim == 1:
+                    n_subset = x[0].shape[0]
+                else:
+                    n_subset = x[0].shape[-2]
+            elif x.ndim == 1:
+                n_subset = x.shape[0]
+            else:
+                n_subset = x.shape[-2]
         if (
             ensemble
-            and hasattr(self.feature_library, "weak_form")
-            and self.feature_library.weak_form
+            and isinstance(self.feature_library, WeakPDELibrary)
             and self.feature_library.is_uniform
         ):
             raise ValueError(
@@ -330,9 +335,76 @@ class SINDy(BaseEstimator):
             raise ValueError("n_models must be a positive integer")
         if (n_subset is not None) and n_subset <= 0:
             raise ValueError("n_subset must be a positive integer")
+        pde_libraries = False
+        weak_libraries = False
+
+        if isinstance(self.feature_library, WeakPDELibrary):
+            self.feature_library.old_x = np.copy(x)
+        if x_dot is None:
+            if isinstance(self.feature_library, WeakPDELibrary):
+                if multiple_trajectories:
+                    x_dot = [
+                        convert_u_dot_integral(xi, self.feature_library) for xi in x
+                    ]
+                else:
+                    x_dot = convert_u_dot_integral(x, self.feature_library)
+            elif isinstance(self.feature_library, PDELibrary):
+                if multiple_trajectories and isinstance(t, Sequence):
+                    x_dot = [
+                        FiniteDifference(d=1, axis=-2)._differentiate(xi, t=ti)
+                        for xi, ti in zip(x, t)
+                    ]
+                elif multiple_trajectories:
+                    x_dot = [
+                        FiniteDifference(d=1, axis=-2)._differentiate(xi, t=t)
+                        for xi in x
+                    ]
+                else:
+                    x_dot = FiniteDifference(d=1, axis=-2)._differentiate(x, t=t)
+
+            elif isinstance(self.feature_library, GeneralizedLibrary):
+                for lib in self.feature_library.libraries_:
+                    if isinstance(lib, WeakPDELibrary):
+                        weak_libraries = True
+                    if isinstance(lib, PDELibrary):
+                        pde_libraries = True
+                if weak_libraries:
+                    if multiple_trajectories:
+                        x_dot = [
+                            convert_u_dot_integral(
+                                xi, self.feature_library.libraries_[0]
+                            )
+                            for xi in x
+                        ]
+                    else:
+                        x_dot = convert_u_dot_integral(
+                            x, self.feature_library.libraries_[0]
+                        )
+                elif pde_libraries:
+                    if multiple_trajectories and isinstance(t, Sequence):
+                        x_dot = [
+                            FiniteDifference(d=1, axis=-2)._differentiate(xi, t=ti)
+                            for xi, ti in zip(x, t)
+                        ]
+                    elif multiple_trajectories:
+                        x_dot = [
+                            FiniteDifference(d=1, axis=-2)._differentiate(xi, t=t)
+                            for xi in x
+                        ]
+                    else:
+                        x_dot = FiniteDifference(d=1, axis=-2)._differentiate(x, t=t)
+
         if multiple_trajectories:
+            self.feature_library.num_trajectories = len(x)
+            if isinstance(self.feature_library, GeneralizedLibrary):
+                for lib in self.feature_library.libraries_:
+                    if isinstance(lib, PDELibrary):
+                        lib.num_trajectories = len(x)
+                    if isinstance(lib, WeakPDELibrary):
+                        lib.num_trajectories = len(x)
             x, x_dot = self._process_multiple_trajectories(x, t, x_dot)
         else:
+            self.feature_library.num_trajectories = 1
             x = validate_input(x, t)
 
             if self.discrete_time:
@@ -344,7 +416,9 @@ class SINDy(BaseEstimator):
             else:
                 if x_dot is None:
                     x_dot = self.differentiation_method(x, t)
-                else:
+                elif (not isinstance(self.feature_library, WeakPDELibrary)) and (
+                    not weak_libraries
+                ):
                     x_dot = validate_input(x_dot, t)
 
         # Set ensemble variables
@@ -353,10 +427,10 @@ class SINDy(BaseEstimator):
 
         # Append control variables
         if self.n_control_features_ > 0:
-            x = concatenate((x, u), axis=1)
+            x = np.concatenate((x, u), axis=1)
 
         # Drop rows where derivative isn't known unless using weak PDE form
-        if not hasattr(self.feature_library, "weak_form"):
+        if not isinstance(self.feature_library, WeakPDELibrary):
             x, x_dot = drop_nan_rows(x, x_dot)
 
         if hasattr(self.optimizer, "unbias"):
@@ -366,42 +440,52 @@ class SINDy(BaseEstimator):
         steps = [("features", self.feature_library), ("model", optimizer)]
         self.model = Pipeline(steps)
 
-        if hasattr(self.feature_library, "temporal_grid"):
-            tgrid = copy(self.feature_library.temporal_grid)
-        else:
-            tgrid = None
-
         action = "ignore" if quiet else "default"
         with warnings.catch_warnings():
             warnings.filterwarnings(action, category=ConvergenceWarning)
             warnings.filterwarnings(action, category=LinAlgWarning)
             warnings.filterwarnings(action, category=UserWarning)
-            pde_library_flag = False
+            pde_library_flag = None
             if isinstance(self.feature_library, PDELibrary):
                 if self.feature_library.spatial_grid is not None:
-                    pde_library_flag = True
+                    pde_library_flag = "PDE"
+            if isinstance(self.feature_library, WeakPDELibrary):
+                if self.feature_library.spatiotemporal_grid is not None:
+                    pde_library_flag = "WeakPDE"
+
             if ensemble and not library_ensemble:
                 self.coef_list = []
+                # save the grid
+                if pde_library_flag == "WeakPDE":
+                    old_spatiotemporal_grid = self.feature_library.spatiotemporal_grid
                 for i in range(n_models):
                     x_ensemble, x_dot_ensemble = drop_random_rows(
                         x,
                         x_dot,
                         n_subset,
                         replace,
-                        tgrid,
                         self.feature_library,
                         pde_library_flag,
+                        multiple_trajectories,
                     )
                     self.model.fit(x_ensemble, x_dot_ensemble)
                     self.coef_list.append(self.model.steps[-1][1].coef_)
+
+                    # reset the grid
+                    if pde_library_flag == "WeakPDE":
+                        self.feature_library.spatiotemporal_grid = (
+                            old_spatiotemporal_grid
+                        )
+                        self.feature_library._set_up_grids()
+
             elif library_ensemble and not ensemble:
                 self.feature_library.library_ensemble = True
                 (self.feature_library).fit(x)
                 n_output_features = self.feature_library.n_output_features_
                 self.coef_list = []
                 for i in range(n_models):
-                    self.feature_library.ensemble_indices = sort(
-                        numpy.random.choice(
+                    self.feature_library.ensemble_indices = np.sort(
+                        np.random.choice(
                             range(n_output_features),
                             n_candidates_to_drop,
                             replace=False,
@@ -410,7 +494,7 @@ class SINDy(BaseEstimator):
                     self.model.fit(x, x_dot)
                     coef_partial = self.model.steps[-1][1].coef_
                     for j in range(n_candidates_to_drop):
-                        coef_partial = insert(
+                        coef_partial = np.insert(
                             coef_partial,
                             self.feature_library.ensemble_indices[j],
                             0,
@@ -428,13 +512,13 @@ class SINDy(BaseEstimator):
                         x_dot,
                         n_subset,
                         replace,
-                        tgrid,
                         self.feature_library,
-                        isinstance(self.feature_library, PDELibrary),
+                        pde_library_flag,
+                        multiple_trajectories,
                     )
                     for j in range(n_models):
-                        self.feature_library.ensemble_indices = sort(
-                            numpy.random.choice(
+                        self.feature_library.ensemble_indices = np.sort(
+                            np.random.choice(
                                 range(n_output_features),
                                 n_candidates_to_drop,
                                 replace=False,
@@ -443,7 +527,7 @@ class SINDy(BaseEstimator):
                         self.model.fit(x_ensemble, x_dot_ensemble)
                         coef_partial = self.model.steps[-1][1].coef_
                         for k in range(n_candidates_to_drop):
-                            coef_partial = insert(
+                            coef_partial = np.insert(
                                 coef_partial,
                                 self.feature_library.ensemble_indices[k],
                                 0,
@@ -456,7 +540,7 @@ class SINDy(BaseEstimator):
         # Get average coefficients if ensembling was used
         if ensemble or library_ensemble:
             if ensemble_aggregator is None:
-                self.model.coef_ = median(self.coef_list, axis=0)
+                self.model.coef_ = np.median(self.coef_list, axis=0)
             else:
                 self.model_coef_ = ensemble_aggregator(self.coef_list)
 
@@ -515,25 +599,40 @@ class SINDy(BaseEstimator):
                     " not used when the model was fit"
                 )
             if multiple_trajectories:
+                x_shapes = []
+                for xi in x:
+                    x_shapes.append(xi.shape)
                 x = [validate_input(xi) for xi in x]
-                return [self.model.predict(xi) for xi in x]
+                return [
+                    self.model.predict(xi).reshape(x_shapes[i])
+                    for i, xi in enumerate(x)
+                ]
             else:
+                x_shape = np.shape(x)
                 x = validate_input(x)
-                return self.model.predict(x)
+                return self.model.predict(x).reshape(x_shape)
         else:
             if multiple_trajectories:
+                x_shapes = []
+                for xi in x:
+                    x_shapes.append(xi.shape)
                 x = [validate_input(xi) for xi in x]
                 u = validate_control_variables(
                     x, u, multiple_trajectories=True, return_array=False
                 )
                 return [
-                    self.model.predict(concatenate((xi, ui), axis=1))
-                    for xi, ui in zip(x, u)
+                    self.model.predict(np.concatenate((xi, ui), axis=1)).reshape(
+                        x_shapes[i]
+                    )
+                    for i, (xi, ui) in enumerate(zip(x, u))
                 ]
             else:
+                x_shape = np.shape(x)
                 x = validate_input(x)
                 u = validate_control_variables(x, u)
-                return self.model.predict(concatenate((x, u), axis=1))
+                return self.model.predict(np.concatenate((x, u), axis=1)).reshape(
+                    x_shape
+                )
 
     def equations(self, precision=3):
         """
@@ -676,7 +775,7 @@ class SINDy(BaseEstimator):
                 x, t, x_dot, return_array=True
             )
         else:
-            if not hasattr(self.feature_library, "weak_form"):
+            if not isinstance(self.feature_library, WeakPDELibrary):
                 x = validate_input(x, t)
             if x_dot is None:
                 if self.discrete_time:
@@ -685,18 +784,16 @@ class SINDy(BaseEstimator):
                 else:
                     x_dot = self.differentiation_method(x, t)
 
-        if ndim(x_dot) == 1:
+        if x_dot.ndim == 1:
             x_dot = x_dot.reshape(-1, 1)
 
         # Append control variables
         if u is not None and self.n_control_features_ > 0:
-            x = concatenate((x, u), axis=1)
+            x = np.concatenate((x, u), axis=1)
 
         # Drop rows where derivative isn't known (usually endpoints)
-        if not hasattr(self.feature_library, "weak_form"):
+        if not isinstance(self.feature_library, WeakPDELibrary):
             x, x_dot = drop_nan_rows(x, x_dot)
-        else:
-            self.feature_library.temporal_grid = t
 
         x_dot_predict = self.model.predict(x)
         return metric(x_dot, x_dot_predict, **metric_kws)
@@ -743,7 +840,6 @@ class SINDy(BaseEstimator):
         """
         if not isinstance(x, Sequence):
             raise TypeError("Input x must be a list")
-
         if self.discrete_time:
             x = [validate_input(xi) for xi in x]
             if x_dot is None:
@@ -777,10 +873,11 @@ class SINDy(BaseEstimator):
                     x_dot = [validate_input(xd, ti) for xd, ti in zip(x_dot, t)]
                 else:
                     x = [validate_input(xi, t) for xi in x]
-                    x_dot = [validate_input(xd, t) for xd in x_dot]
+                    if not isinstance(self.feature_library, WeakPDELibrary):
+                        x_dot = [validate_input(xd, t) for xd in x_dot]
 
         if return_array:
-            return vstack(x), vstack(x_dot)
+            return np.vstack(x), np.vstack(x_dot)
         else:
             return x, x_dot
 
@@ -934,16 +1031,16 @@ class SINDy(BaseEstimator):
 
             # New version of sklearn changes attribute name
             if float(__version__[:3]) >= 1.0:
-                x = zeros((t, self.n_features_in_ - self.n_control_features_))
+                x = np.zeros((t, self.n_features_in_ - self.n_control_features_))
             else:
-                x = zeros((t, self.n_input_features_ - self.n_control_features_))
+                x = np.zeros((t, self.n_input_features_ - self.n_control_features_))
             x[0] = x0
 
             if u is None or self.n_control_features_ == 0:
                 if u is not None:
                     warnings.warn(
-                        "Control variables u were ignored because control variables"
-                        " were not used when the model was fit"
+                        "Control variables u were ignored because control "
+                        "variables were not used when the model was fit"
                     )
                 for i in range(1, t):
                     x[i] = self.predict(x[i - 1 : i])
@@ -951,12 +1048,12 @@ class SINDy(BaseEstimator):
                         return x[: i + 1]
             else:
                 for i in range(1, t):
-                    x[i] = self.predict(x[i - 1 : i], u=u[i - 1, newaxis])
+                    x[i] = self.predict(x[i - 1 : i], u=u[i - 1, np.newaxis])
                     if check_stop_condition(x[i]):
                         return x[: i + 1]
             return x
         else:
-            if isscalar(t):
+            if np.isscalar(t):
                 raise ValueError(
                     "For continuous time model, t must be an array of time"
                     " points at which to simulate"
@@ -965,12 +1062,12 @@ class SINDy(BaseEstimator):
             if u is None or self.n_control_features_ == 0:
                 if u is not None:
                     warnings.warn(
-                        "Control variables u were ignored because control variables"
-                        " were not used when the model was fit"
+                        "Control variables u were ignored because control "
+                        "variables were not used when the model was fit"
                     )
 
                 def rhs(t, x):
-                    return self.predict(x[newaxis, :])[0]
+                    return self.predict(x[np.newaxis, :])[0]
 
             else:
                 if not callable(u):
@@ -983,22 +1080,24 @@ class SINDy(BaseEstimator):
 
                     t = t[:-1]
                     warnings.warn(
-                        "Last time point dropped in simulation because interpolation"
-                        " of control input was used. To avoid this, pass in a callable"
-                        " for u."
+                        "Last time point dropped in simulation because "
+                        "interpolation of control input was used. To avoid "
+                        "this, pass in a callable for u."
                     )
                 else:
                     u_fun = u
 
-                if ndim(u_fun(t[0])) == 1:
+                if u_fun(t[0]).ndim == 1:
 
                     def rhs(t, x):
-                        return self.predict(x[newaxis, :], u_fun(t).reshape(1, -1))[0]
+                        return self.predict(x[np.newaxis, :], u_fun(t).reshape(1, -1))[
+                            0
+                        ]
 
                 else:
 
                     def rhs(t, x):
-                        return self.predict(x[newaxis, :], u_fun(t))[0]
+                        return self.predict(x[np.newaxis, :], u_fun(t))[0]
 
             # Need to hard-code below, because odeint and solve_ivp
             # have different syntax and integration options.
